@@ -29,22 +29,22 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 /*
- * Copyright (c) 2010-2012 Yuichi Watanabe
+ * Copyright (c) 2010-2014 Yuichi Watanabe
  */
 .
 
 printhelp(){
-	echo "usage: $0 [-f] [-l lba1] [-i lba2] [-c cmdline] device [loader] [vmm] [module1]"
+	echo "usage: $0 [-f] [-l lba1] [-i lba2] [-c cmdline] [-b bootloader] [-v vmm] [-m module1] device"
 	echo "       $0 -s [-l lba1] device"
 	echo '  -f         first time (do not check existing data)'
 	echo '  -l lba1    a lba where a boot record is installed (default 0; MBR)'
 	echo '  -i lba2    a lba where a image is installed (default 1)'
 	echo '  -c         cmdline boot parameters to vmm'
 	echo '  -s         show the installing status'
+	echo '  -b bootloader     a path to bootloader (default: bootloader/bootloader)'
+	echo '  -v vmm        a path to vmm.elf (default: vmm.elf)'
+	echo '  -m module1    a path to guest bios (default: bios.bin)'
 	echo '  device     write to device (ex. /dev/sda)'
-	echo '  loader     a path to bootloader (default: bootloader/bootloader)'
-	echo '  vmm        a path to vmm.elf (default: vmm/vmm/vmm.elf)'
-	echo '  module1    a path to guest bios (default: bios/out/bios.bin)'
 }
 
 show(){
@@ -126,14 +126,43 @@ generatembr(){
 	dd if="$loader" bs=1 skip=510 count=2 2> /dev/null			# 510-   2 bytes
 }
 
+getbsssize(){
+	bsssize=0
+	if ! dd if="$vmm" bs=1 skip=0 count=4 2> /dev/null | od |
+		grep -q '^0000000 042577 043114'
+	then
+		echo "ELF header not found in \`$vmm'." >&2
+		exit 1
+	fi
+	set -- $(dd if="$vmm" bs=1 skip=28 count=4 2> /dev/null | od -i)
+	phoff=$2
+	set -- $(dd if="$vmm" bs=1 skip=42 count=2 2> /dev/null | od -i)
+	phentsize=$2
+	set -- $(dd if="$vmm" bs=1 skip=44 count=2 2> /dev/null | od -i)
+	phnum=$2
+	while test $phnum -gt 0
+	do
+		phnum=$(($phnum-1))
+		set -- $(($phoff+$phentsize*$phnum))
+		set -- $(($1+16)) $(dd if="$vmm" bs=1 skip=$1 count=4 2> /dev/null | od -i)
+		case "$3" in
+		1)	;;
+		*)	continue;;
+		esac
+		set -- $(($1+4)) $(dd if="$vmm" bs=1 skip=$1 count=4 2> /dev/null | od -i)
+		set -- "$3" $(dd if="$vmm" bs=1 skip=$1 count=4 2> /dev/null | od -i)
+		bsssize=$(($bsssize+$3-$1))
+	done
+}
+
 first=0
 device=
 lba1=0
 lba2=1
 loader=bootloader/bootloader
-vmm=vmm/vmm.elf
+vmm=vmm.elf
 cmdline=
-module1=bios/out/bios.bin
+module1=bios.bin
 arg=.
 show=0
 
@@ -151,14 +180,20 @@ do
 	-c)
 		shift;
 		cmdline=$1;;
+	-b)
+		shift;
+		boader=$1;;
+	-v)
+		shift;
+		vmm=$1;;
+	-m)
+		shift;
+		module1=$1;;
 	-*)	echo "unrecognized option \`$1'" >&2
 		exit 1;;
 	*)	case $arg in
 		.)		device="$1";;
-		..)		loader="$1";;
-		...)		vmm="$1";;
-		....)		module1="$1";;
-		.....)		echo "extra operand \`$1'" >&2
+		..)		echo "extra operand \`$1'" >&2
 				exit 1;;
 		esac
 		arg=.$arg;;
@@ -195,10 +230,9 @@ case $first in
 	fi;;
 esac
 
-size0=`cat "$vmm" | wc -c`
-cmdlinesize=0
-size1=0
-cmdlineoff=0
+getbsssize
+
+size0=$((`cat "$vmm" | wc -c`+$bsssize))
 off1=0
 total=$size0
 if [ -n "$cmdline" ]; then
@@ -218,12 +252,21 @@ generatembr $total $cmdlineoff $off1 | dd of="$device" seek="$lba1" conv=notrunc
 
 echo "Writing to lba $lba2"
 (
-    cat "$vmm"
-    if [ -n "$cmdline" ]; then
-	    echo -n $cmdline;
-	    echo -n -e '\000'; # octal value
-    fi
-    if [ -n "$module1" ]; then
-	    cat "$module1"
-    fi
+	cat "$vmm"
+	if test $bsssize -ge 512
+	then
+		dd if=/dev/zero count=$(($bsssize/512)) 2>/dev/null
+		bsssize=$(($bsssize%512))
+	fi
+	if test $bsssize -gt 0
+	then
+		dd if=/dev/zero bs=1 count=$bsssize 2>/dev/null
+	fi
+	if [ -n "$cmdline" ]; then
+		echo -n $cmdline;
+		echo -n -e '\000'; # octal value
+	fi
+	if [ -n "$module1" ]; then
+		cat "$module1"
+	fi
 ) | dd of="$device" seek="$lba2" conv=notrunc 2> /dev/null
