@@ -33,6 +33,16 @@
 #ifndef _CORE_ACPI_H
 #define _CORE_ACPI_H
 
+#include <core/types.h>
+
+struct acpi_vm_data {
+	bool iopass;
+};
+
+struct acpi_data {
+	bool smi_hook_disabled;
+};
+
 struct acpi_sleep {
 	bool valid;
 	char type[2];
@@ -40,7 +50,12 @@ struct acpi_sleep {
 
 extern struct acpi_sleep acpi_sleep[6];
 
-void acpi_modify_aml(u8 *start, u8 *end);
-void acpi_poweroff(void);
+void acpi_iohook (void);
+void acpi_poweroff (void);
+bool get_acpi_time_raw (u32 *r);
+void acpi_smi_hook (void);
+void acpi_reset (void);
+void acpi_load_aml(u8 *start, u8 *end);
+void acpi_parse_aml(void);
 
-#endif /* _ACPI_H */
+#endif
