@@ -1,5 +1,5 @@
 /*
- * From NetBSD usr/src/usr.sbin/acpitools/amldb/amldb.c
+ * From NetBSD 9.2 usr/src/usr.sbin/acpitools/amldb/amldb.c
  */
 /*-
  * Copyright (c) 1999 Mitsuru IWASAKI <iwasaki@FreeBSD.org>
@@ -47,7 +47,7 @@
 #include "aml/aml_parse.h"
 #include "aml/aml_region.h"
 
-static void
+void
 acpi_load_aml(u8 *start, u8 *end)
 {
 	struct aml_environ	env;
@@ -207,12 +207,9 @@ parse_sx(void)
 					"_S5_", sx_callback);
 }
 
-
 void
-acpi_modify_aml(u8 *start, u8 *end)
+acpi_parse_aml(void)
 {
-	acpi_load_aml(start, end);
-
 	parse_sx();
 #ifdef TTY_SERIAL
 	disable_pnp0501_dis();
