@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2013 Yuichi Watanabe
+ * Copyright (c) 2012-2014 Yuichi Watanabe
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -71,12 +71,12 @@ parse_apic_structure(struct apic_header *apic_header)
 	}
 }
 
-static acpi_parser_t parse_madt;
 static void
-parse_madt(void *table, u32 length)
+parse_madt(struct acpi_desc_header *table)
 {
 	struct acpi_madt	*acpi_madt = (struct acpi_madt *)table;
 	struct apic_header	*apic_header;
+	u32			length = table->length;
 
 	for (apic_header = (struct apic_header *)(acpi_madt + 1);
 	     (ulong)apic_header < (ulong)acpi_madt + length;
@@ -91,10 +91,15 @@ parse_madt(void *table, u32 length)
 static void
 acpi_madt_overwrite(void)
 {
+	struct acpi_desc_header *table;
+
 	if (vm_get_id() != 0) {
 		return;
 	}
-	acpi_register_parser(ACPI_MADT_SIGNATURE, parse_madt);
+	table = acpi_find_table(ACPI_MADT_SIGNATURE);
+	if (table) {
+		parse_madt(table);
+	}
 }
 
-INITFUNC("setupvm4", acpi_madt_overwrite);
+INITFUNC("vminit1", acpi_madt_overwrite);
