@@ -1,11 +1,11 @@
 #ifndef _AML_COMPAT_H
 #define _AML_COMPAT_H
 
-#include <common/common.h>
+#include <common/calc.h>
+#include <common/string.h>
 #include <core/assert.h>
 #include <core/mm.h>
-#include <core/string.h>
-#include <io/io.h>
+#include <io.h>
 #include "aml_common.h"
 
 #define bcopy(src, dest, len)\
