@@ -36,39 +36,52 @@
 #define CPUID_1				0x1
 #define CPUID_1_EBX_NUMOFLP_MASK	0x00FF0000
 #define CPUID_1_EBX_NUMOFLP_1		0x00010000
-#define CPUID_1_ECX_MONITOR_BIT		0x8
 #define CPUID_1_ECX_VMX_BIT		0x20
+#define CPUID_1_ECX_PCID_BIT		0x20000
 #define CPUID_1_EDX_PSE_BIT		0x8
 #define CPUID_1_EDX_TSC_BIT		0x10
 #define CPUID_1_EDX_MSR_BIT		0x20
 #define CPUID_1_EDX_PAE_BIT		0x40
 #define CPUID_1_EDX_APIC_BIT		0x200
 #define CPUID_1_EDX_SEP_BIT		0x800
+#define CPUID_1_EDX_MTRR_BIT		0x1000
+#define CPUID_1_EDX_PAT_BIT		0x10000
 #define CPUID_4_EAX_NUMOFTHREADS_MASK	0x03FFC000
 #define CPUID_4_EAX_NUMOFCORES_MASK	0xFC000000
 #define CPUID_EXT_0			0x80000000
 #define CPUID_EXT_1			0x80000001
 #define CPUID_EXT_1_ECX_SVM_BIT		0x4
+#define CPUID_EXT_1_EDX_PAGE1GB_BIT	0x4000000
+#define CPUID_EXT_1_EDX_64_BIT		0x20000000
+#define CPUID_EXT_7			0x80000007
+#define CPUID_EXT_7_EDX_TSCINVARIANT_BIT	0x100
+#define CPUID_EXT_8			0x80000008
+#define CPUID_EXT_8_EAX_PHYSADDRSIZE_MASK	0xFF
 #define CPUID_EXT_A			0x8000000A
 #define CPUID_EXT_A_EDX_NP_BIT		0x1
 #define CPUID_EXT_A_EDX_SVM_LOCK_BIT	0x4
-
+#define CPUID_EXT_A_EDX_FLUSH_BY_ASID_BIT	0x40
 #define MSR_IA32_TIME_STAMP_COUNTER	0x10
-#define MSR_IA32_APIC_BASE		0x1B
-#define MSR_IA32_APIC_BASE_APIC_GLOBAL_ENABLE_BIT	0x800
-#define MSR_IA32_APIC_BASE_APIC_BASE_MASK	0xFFFFFF000ULL
+#define MSR_IA32_APIC_BASE_MSR		0x1B
+#define MSR_IA32_APIC_BASE_MSR_APIC_GLOBAL_ENABLE_BIT	0x800
+#define MSR_IA32_APIC_BASE_MSR_APIC_BASE_MASK	0xFFFFFF000ULL
 #define MSR_IA32_FEATURE_CONTROL	0x3A
 #define MSR_IA32_FEATURE_CONTROL_LOCK_BIT	0x1
 #define MSR_IA32_FEATURE_CONTROL_VMXON_BIT	0x4
 #define MSR_IA32_BIOS_UPDT_TRIG		0x79
-#define MSR_IA32_MTRRCAP		0xfe
-#define  MSR_IA32_MTRRCAP_VCNT_MASK		0xff
-#define  MSR_IA32_MTRRCAP_FIX_BIT		0x100
+#define MSR_IA32_MTRRCAP		0xFE
+#define MSR_IA32_MTRRCAP_VCNT_MASK	0xFF
+#define MSR_IA32_MTRRCAP_FIX_BIT	0x100
+#define MSR_IA32_MTRRCAP_WC_BIT		0x400
 #define MSR_IA32_SYSENTER_CS		0x174
 #define MSR_IA32_SYSENTER_ESP		0x175
 #define MSR_IA32_SYSENTER_EIP		0x176
 #define MSR_IA32_MTRR_PHYSBASE0		0x200
+#define MSR_IA32_MTRR_PHYSBASE0_TYPE_MASK	0xFFULL
+#define MSR_IA32_MTRR_PHYSBASE0_PHYSBASE_MASK	0xFFFFFFFFFF000ULL
 #define MSR_IA32_MTRR_PHYSMASK0		0x201
+#define MSR_IA32_MTRR_PHYSMASK0_V_BIT	0x800ULL
+#define MSR_IA32_MTRR_PHYSMASK0_PHYSMASK_MASK	0xFFFFFFFFFF000ULL
 #define MSR_IA32_MTRR_PHYSBASE1		0x202
 #define MSR_IA32_MTRR_PHYSMASK1		0x203
 #define MSR_IA32_MTRR_PHYSBASE2		0x204
@@ -77,42 +90,35 @@
 #define MSR_IA32_MTRR_PHYSMASK3		0x207
 #define MSR_IA32_MTRR_PHYSBASE4		0x208
 #define MSR_IA32_MTRR_PHYSMASK4		0x209
-#define MSR_IA32_MTRR_PHYSBASE5		0x20a
-#define MSR_IA32_MTRR_PHYSMASK5		0x20b
-#define MSR_IA32_MTRR_PHYSBASE6		0x20c
-#define MSR_IA32_MTRR_PHYSMASK6		0x20d
-#define MSR_IA32_MTRR_PHYSBASE7		0x20e
-#define MSR_IA32_MTRR_PHYSMASK7		0x20f
+#define MSR_IA32_MTRR_PHYSBASE5		0x20A
+#define MSR_IA32_MTRR_PHYSMASK5		0x20B
+#define MSR_IA32_MTRR_PHYSBASE6		0x20C
+#define MSR_IA32_MTRR_PHYSMASK6		0x20D
+#define MSR_IA32_MTRR_PHYSBASE7		0x20E
+#define MSR_IA32_MTRR_PHYSMASK7		0x20F
 #define MSR_IA32_MTRR_PHYSBASE8		0x210
 #define MSR_IA32_MTRR_PHYSMASK8		0x211
 #define MSR_IA32_MTRR_PHYSBASE9		0x212
 #define MSR_IA32_MTRR_PHYSMASK9		0x213
-#define  MSR_IA32_MTRR_PHYSMASK_V_BIT		0x800
-#define  MSR_IA32_MTRR_PHYSMASK_MASK_MASK	0xfffffffffffff000
-#define  MSR_IA32_MTRR_PHYSBASE_TYPE_MASK	0xff
-#define  MSR_IA32_MTRR_PHYSBASE_BASE_MASK	0xfffffffffffff000
-#define MSR_IA32_MTRR_FIX64K_000	0x250
-#define MSR_IA32_MTRR_FIX16K_800	0x258
-#define MSR_IA32_MTRR_FIX16K_A00	0x259
-#define MSR_IA32_MTRR_FIX4K_C000	0x268
-#define MSR_IA32_MTRR_FIX4K_C800	0x269
-#define MSR_IA32_MTRR_FIX4K_D000	0x26a
-#define MSR_IA32_MTRR_FIX4K_D800	0x26b
-#define MSR_IA32_MTRR_FIX4K_E000	0x26c
-#define MSR_IA32_MTRR_FIX4K_E800	0x26d
-#define MSR_IA32_MTRR_FIX4K_F000	0x26e
-#define MSR_IA32_MTRR_FIX4K_F800	0x26f
-#define  MSR_IA32_MTRR_FIX_TYPE_MASK		0xff
-#define  MSR_IA32_MTRR_FIX_TYPE_SHIFT		8
-#define  MSR_IA32_MTRR_FIX_TYPE_NUM		8
+#define MSR_IA32_MTRR_FIX64K_00000	0x250
+#define MSR_IA32_MTRR_FIX16K_80000	0x258
+#define MSR_IA32_MTRR_FIX16K_A0000	0x259
+#define MSR_IA32_MTRR_FIX4K_C0000	0x268
+#define MSR_IA32_MTRR_FIX4K_C8000	0x269
+#define MSR_IA32_MTRR_FIX4K_D0000	0x26A
+#define MSR_IA32_MTRR_FIX4K_D8000	0x26B
+#define MSR_IA32_MTRR_FIX4K_E0000	0x26C
+#define MSR_IA32_MTRR_FIX4K_E8000	0x26D
+#define MSR_IA32_MTRR_FIX4K_F0000	0x26E
+#define MSR_IA32_MTRR_FIX4K_F8000	0x26F
 #define MSR_IA32_PAT			0x277
 #define  MSR_DATA_PAT_COUNT 8
 #define  MSR_PAT_FIELD(msrval, index) \
 	 ( ( (msrval) >> (8 * (index)) ) & 0xff );
-#define MSR_IA32_MTRR_DEF_TYPE		0x2ff
-#define  MSR_IA32_MTRR_DEF_TYPE_TYPE_MASK	0x0ff
-#define  MSR_IA32_MTRR_DEF_TYPE_FE_BIT		0x400
-#define  MSR_IA32_MTRR_DEF_TYPE_E_BIT		0x800
+#define MSR_IA32_MTRR_DEF_TYPE		0x2FF
+#define MSR_IA32_MTRR_DEF_TYPE_TYPE_MASK	0xFFULL
+#define MSR_IA32_MTRR_DEF_TYPE_FE_BIT	0x400ULL
+#define MSR_IA32_MTRR_DEF_TYPE_E_BIT	0x800ULL
 #define MSR_IA32_VMX_BASIC		0x480
 #define MSR_IA32_VMX_PINBASED_CTLS	0x481
 #define MSR_IA32_VMX_PROCBASED_CTLS	0x482
@@ -122,7 +128,14 @@
 #define MSR_IA32_VMX_CR0_FIXED1		0x487
 #define MSR_IA32_VMX_CR4_FIXED0		0x488
 #define MSR_IA32_VMX_CR4_FIXED1		0x489
-#define MSR_IA32_VMX_PROCBASED_CTLS2	0x48b
+#define MSR_IA32_VMX_PROCBASED_CTLS2	0x48B
+#define MSR_IA32_VMX_EPT_VPID_CAP	0x48C
+#define MSR_IA32_VMX_EPT_VPID_CAP_PAGEWALK_LENGTH_4_BIT	0x40
+#define MSR_IA32_VMX_EPT_VPID_CAP_EPTSTRUCT_WB_BIT	0x4000
+#define MSR_IA32_VMX_EPT_VPID_CAP_INVEPT_BIT	0x100000
+#define MSR_IA32_VMX_EPT_VPID_CAP_INVEPT_ALL_CONTEXT_BIT	0x4000000
+#define MSR_IA32_VMX_EPT_VPID_CAP_INVVPID_BIT	0x100000000ULL
+#define MSR_IA32_VMX_EPT_VPID_CAP_INVVPID_SINGLE_CONTEXT_BIT 0x20000000000ULL
 #define MSR_IA32_EFER			0xC0000080
 #define MSR_IA32_EFER_SCE_BIT		0x1
 #define MSR_IA32_EFER_LME_BIT		0x100
@@ -138,11 +151,19 @@
 #define MSR_IA32_GS_BASE		0xC0000101
 #define MSR_IA32_KERNEL_GS_BASE		0xC0000102
 #define MSR_AMD_VM_CR			0xC0010114
+#define MSR_AMD_VM_CR_R_INIT_BIT	0x2
 #define MSR_AMD_VM_CR_DIS_A20M_BIT	0x4
 #define MSR_AMD_VM_CR_SVMDIS_BIT	0x10
+#define MSR_AMD_SYSCFG			0xC0010010
+#define MSR_AMD_SYSCFG_MTRRFIXDRAMEN_BIT	0x40000
+#define MSR_AMD_SYSCFG_MTRRFIXDRAMMODEN_BIT	0x80000
+#define MSR_AMD_SYSCFG_MTRRTOM2EN_BIT	0x200000
+#define MSR_AMD_SYSCFG_TOM2FORCEMEMTYPEWB_BIT	0x400000
+#define MSR_AMD_TOP_MEM2		0xC001001D
+#define MSR_AMD_TOP_MEM2_ADDR_MASK	0xFFFFFFF800000ULL
 #define MSR_AMD_VM_HSAVE_PA		0xC0010117
 
-/* 16-Bit Control Fields */
+/* 16-Bit Control Field */
 #define VMCS_VPID			0x0
 
 /* 16-Bit Guest-State Fields */
@@ -183,8 +204,12 @@
 #define VMCS_TSC_OFFSET_HIGH		0x2011
 #define VMCS_VIRT_APIC_PAGE_ADDR	0x2012
 #define VMCS_VIRT_APIC_PAGE_ADDR_HIGH	0x2013
-#define VMCS_EPT_POINTER_ADDR		0x201A
-#define VMCS_EPT_POINTER_ADDR_HIGH	0x201B
+#define VMCS_EPT_POINTER		0x201A
+#define VMCS_EPT_POINTER_HIGH		0x201B
+
+/* 64-Bit Read-Only Data Field */
+#define VMCS_GUEST_PHYSICAL_ADDRESS	0x2400
+#define VMCS_GUEST_PHYSICAL_ADDRESS_HIGH	0x2401
 
 /* 64-Bit Data Field */
 #define VMCS_GUEST_PHYS_ADDR		0x2400
@@ -197,10 +222,22 @@
 #define VMCS_GUEST_IA32_DEBUGCTL_HIGH	0x2803
 #define VMCS_GUEST_IA32_PAT		0x2804
 #define VMCS_GUEST_IA32_PAT_HIGH	0x2805
+#define VMCS_GUEST_IA32_EFER		0x2806
+#define VMCS_GUEST_IA32_EFER_HIGH	0x2807
+#define VMCS_GUEST_PDPTE0		0x280A
+#define VMCS_GUEST_PDPTE0_HIGH		0x280B
+#define VMCS_GUEST_PDPTE1		0x280C
+#define VMCS_GUEST_PDPTE1_HIGH		0x280D
+#define VMCS_GUEST_PDPTE2		0x280E
+#define VMCS_GUEST_PDPTE2_HIGH		0x280F
+#define VMCS_GUEST_PDPTE3		0x2810
+#define VMCS_GUEST_PDPTE3_HIGH		0x2811
 
 /* 64-Bit Host-State Fields */
-#define VMCS_HOST_IA32_PAT		0x2c00
-#define VMCS_HOST_IA32_PAT_HIGH		0x2c01
+#define VMCS_HOST_IA32_PAT		0x2C00
+#define VMCS_HOST_IA32_PAT_HIGH		0x2C01
+#define VMCS_HOST_IA32_EFER		0x2C02
+#define VMCS_HOST_IA32_EFER_HIGH	0x2C03
 
 /* 32-Bit Control Fields */
 #define VMCS_PIN_BASED_VMEXEC_CTL	0x4000
@@ -213,7 +250,6 @@
 #define VMCS_VMEXIT_MSR_STORE_COUNT	0x400E
 #define VMCS_VMEXIT_MSR_LOAD_COUNT	0x4010
 #define VMCS_VMENTRY_CTL		0x4012
-#define VMCS_VMENTRY_CTL_64_GUEST_BIT	0x200
 #define VMCS_VMENTRY_MSR_LOAD_COUNT	0x4014
 #define VMCS_VMENTRY_INTR_INFO_FIELD	0x4016
 #define VMCS_VMENTRY_EXCEPTION_ERRCODE	0x4018
@@ -311,10 +347,6 @@
 #define VMCS_HOST_RSP			0x6C14
 #define VMCS_HOST_RIP			0x6C16
 
-/* Bit-fields */
-#define VMCS_VMEXIT_CTL_HOST_ADDRESS_SPACE_SIZE_BIT	0x200
-#define VMCS_VMEXIT_CTL_LOAD_IA32_PAT_BIT		0x80000
-#define VMCS_VMENTRY_CTL_LOAD_IA32_PAT_BIT		0x4000
 #define VMCS_GUEST_INTERRUPTIBILITY_STATE_BLOCKING_BY_STI_BIT		0x1
 #define VMCS_GUEST_INTERRUPTIBILITY_STATE_BLOCKING_BY_MOV_SS_BIT	0x2
 #define VMCS_GUEST_INTERRUPTIBILITY_STATE_BLOCKING_BY_SMI_BIT		0x4
@@ -339,21 +371,29 @@
 #define VMCS_PROC_BASED_VMEXEC_CTL_USEMSRBMP_BIT	0x10000000
 #define VMCS_PROC_BASED_VMEXEC_CTL_MONITOREXIT_BIT	0x20000000
 #define VMCS_PROC_BASED_VMEXEC_CTL_PAUSEEXIT_BIT	0x40000000
-#define VMCS_PROC_BASED_VMEXEC_CTL_SEC_CTL_BIT		0x80000000
-#define VMCS_PROC_BASED_VMEXEC_CTL2_EPT_BIT		0x2
-#define VMCS_PROC_BASED_VMEXEC_CTL2_VPID_BIT		0x20
-#define VMCS_PROC_BASED_VMEXEC_UNRESTRICTED_GUEST_BIT	0x80
+#define VMCS_PROC_BASED_VMEXEC_CTL_ACTIVATECTLS2_BIT	0x80000000
+#define VMCS_VMEXIT_CTL_HOST_ADDRESS_SPACE_SIZE_BIT 0x200
+#define VMCS_VMEXIT_CTL_SAVE_IA32_PAT_BIT	0x40000
+#define VMCS_VMEXIT_CTL_LOAD_IA32_PAT_BIT	0x80000
+#define VMCS_VMEXIT_CTL_SAVE_IA32_EFER_BIT	0x100000
+#define VMCS_VMEXIT_CTL_LOAD_IA32_EFER_BIT	0x200000
+#define VMCS_VMENTRY_CTL_64_GUEST_BIT	0x200
+#define VMCS_VMENTRY_CTL_LOAD_IA32_PAT_BIT	0x4000
+#define VMCS_VMENTRY_CTL_LOAD_IA32_EFER_BIT	0x8000
+#define VMCS_PROC_BASED_VMEXEC_CTL2_ENABLE_EPT_BIT	0x2
+#define VMCS_PROC_BASED_VMEXEC_CTL2_ENABLE_VPID_BIT	0x20
+#define VMCS_PROC_BASED_VMEXEC_CTL2_UNRESTRICTED_GUEST_BIT	0x80
 #define VMCS_GUEST_ACTIVITY_STATE_ACTIVE	0x0
 #define VMCS_GUEST_ACTIVITY_STATE_HLT		0x1
 #define VMCS_GUEST_ACTIVITY_STATE_SHUTDOWN	0x2
 #define VMCS_GUEST_ACTIVITY_STATE_WAIT_FOR_SIPI	0x3
+#define VMCS_EPT_POINTER_EPT_WB		0x6
+#define VMCS_EPT_PAGEWALK_LENGTH_4	0x18
 
 #define VMXON_REGION_SIZE		0x1000
 #define VMCS_REGION_SIZE		0x1000
 #define ACCESS_RIGHTS_MASK		0xF0FF
 #define ACCESS_RIGHTS_UNUSABLE_BIT	0x10000
-#define ACCESS_RIGHTS_DPL_MASK		0x60
-#define ACCESS_RIGHTS_DPL_SHIFT		5
 #define ACCESS_RIGHTS_P_BIT		0x80
 #define ACCESS_RIGHTS_L_BIT		0x2000
 #define ACCESS_RIGHTS_D_B_BIT		0x4000
@@ -400,11 +440,19 @@
 #define EXIT_REASON_PAUSE		0x28
 #define EXIT_REASON_ENTFAIL_MACHINE_CHK	0x29
 #define EXIT_REASON_TPR_BELOW_THRESHOLD	0x2B
-#define EXIT_REASON_EPT_VIOLATION	0x30
-#define EXIT_REASON_EPT_MISCONFIG	0x31
-#define EXIT_REASON_XSETBV		0x37
 #define EXIT_REASON_VMEXIT_FROM_VMX_ROOT_OPERATION_BIT	0x20000000
 #define EXIT_REASON_VMENTRY_FAILURE_BIT	0x80000000
+#define EXIT_REASON_APIC_ACCESS		0x2C
+#define EXIT_REASON_ACCESS_GDTR_OR_IDTR	0x2E
+#define EXIT_REASON_ACCESS_LDTR_OR_TR	0x2F
+#define EXIT_REASON_EPT_VIOLATION	0x30
+#define EXIT_REASON_EPT_MISCONFIG	0x31
+#define EXIT_REASON_INVEPT		0x32
+#define EXIT_REASON_RDTSCP		0x33
+#define EXIT_REASON_VMX_PREEMPT_TIMER	0x34
+#define EXIT_REASON_INVVPID		0x35
+#define EXIT_REASON_WBINVD		0x36
+#define EXIT_REASON_XSETBV		0x37
 
 #define VMEXIT_CR0_READ			0x0
 #define VMEXIT_CR1_READ			0x1
@@ -604,7 +652,6 @@
 
 #define CR3_PWT_BIT			0x8
 #define CR3_PCD_BIT			0x10
-#define CR3_NO_INVALIDATE		0x8000000000000000
 
 #define CR4_VME_BIT			0x1
 #define CR4_PVI_BIT			0x2
@@ -620,7 +667,10 @@
 #define CR4_VMXE_BIT			0x2000
 #define CR4_PCIDE_BIT			0x20000
 #define CR4_OSXSAVE_BIT			0x40000
-#define CR4_SMEP_BIT			0x100000
+
+#define XCR0_X87_STATE_BIT		0x1
+#define XCR0_SSE_STATE_BIT		0x2
+#define XCR0_AVX_STATE_BIT		0x4
 
 #define PDE_P_BIT			0x1
 #define PDE_RW_BIT			0x2
@@ -645,7 +695,7 @@
 #define PDE_2M_PAT_BIT			PDE_PS_PAT_BIT
 #define PDE_2M_OFFSET_MASK		0x001FFFFF
 #define PDE_2M_ADDR_MASK		(~PDE_2M_OFFSET_MASK)
-#define PDE_ADDR_MASK64			0x000000FFFFFFF000ULL
+#define PDE_ADDR_MASK64			0x0000000FFFFFF000ULL
 
 #define PTE_P_BIT			0x1
 #define PTE_RW_BIT			0x2
@@ -663,10 +713,7 @@
 #define PTE_NX_BIT			0x8000000000000000ULL
 #define PTE_ADDR_MASK			0xFFFFF000
 #define PTE_ATTR_MASK			0xFFF
-#define PTE_ADDR_MASK64			0x000000FFFFFFF000ULL
-
-#define PTE_NUM_IN_TABLE_32		1024
-#define PTE_NUM_IN_TABLE_64		512
+#define PTE_ADDR_MASK64			0x0000000FFFFFF000ULL
 
 #define EXCEPTION_DE			0x0
 #define EXCEPTION_DB			0x1
@@ -687,7 +734,7 @@
 #define EXCEPTION_AC			0x11
 #define EXCEPTION_MC			0x12
 #define EXCEPTION_XM			0x13
-#define EXT_INT_START			0x20
+#define EXCEPTION_SX			0x1E
 
 #define NUM_OF_IOPORT			0x10000
 #define NUM_OF_INT			0x100
@@ -707,12 +754,15 @@
 #define PAGESIZE			0x1000
 #define PAGESIZE2M			0x200000
 #define PAGESIZE4M			0x400000
+#define PAGESIZE1G			0x40000000
 #define PAGESIZE_SHIFT			12
 #define PAGESIZE2M_SHIFT		21
 #define PAGESIZE4M_SHIFT		22
+#define PAGESIZE1G_SHIFT		30
 #define PAGESIZE_MASK			(PAGESIZE - 1)
 #define PAGESIZE2M_MASK			(PAGESIZE2M - 1)
 #define PAGESIZE4M_MASK			(PAGESIZE4M - 1)
+#define PAGESIZE1G_MASK			(PAGESIZE1G - 1)
 
 #define PIT_COUNTER0			0x40
 #define PIT_COUNTER1			0x41
@@ -735,6 +785,6 @@
 #define PIT_CONTROL_COUNTER2		0x80
 #define PIT_CONTROL_READBACK		0xC0
 
-#define VMM_MINSTACKSIZE		1024
+#define VMM_MINSTACKSIZE		3072
 
 #endif
