@@ -27,33 +27,39 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <common/common.h>
+#include <common/calc.h>
 #include <core/assert.h>
 #include <core/cpu.h>
+#include <core/exint.h>
+#include <core/gmm.h>
 #include <core/initfunc.h>
 #include <core/mm.h>
-#include <core/param.h>
+#include <core/pci.h>
 #include <core/printf.h>
-#include <core/sleep.h>
 #include <core/spinlock.h>
-#include <io/pci.h>
+#include <core/string.h>
+#include <core/vm_config.h>
 #include "apic.h"
 #include "acpi.h"
-#include "apic_pass.h"
+#include "callrealmode.h"
+#include "cpu_emul.h"
 #include "current.h"
 #include "gbiosloader.h"
 #include "int.h"
+#include "loadbootsector.h"
+#include "reload_os.h"
 #include "panic.h"
 #include "pcpu.h"
-#include "setup_guestboot.h"
+#include "sleep.h"
 #include "vm.h"
+#include "vmmcall_boot.h"
 
 static size_t driver_data_size = 0;
 static int new_vm_called = 0;
 
-static LIST2_DEFINE_HEAD_INIT(struct vm, vm_list);
-static spinlock_t vm_list_lock = SPINLOCK_INITIALIZER;
-int vm_num = 0;
+static LIST2_DEFINE_HEAD_INIT(vm_list, struct vm, vm_list);
+static struct vm *vm0;
+static u8 bios_boot_drive;
 
 drvdata_hdl_t
 vm_alloc_driver_data(size_t size)
@@ -76,6 +82,19 @@ vm_get_driver_data(drvdata_hdl_t handle)
 	return (void *)((ulong)(current->vm->driver_data) + handle);
 }
 
+void *
+vm_get_driver_data_of_vm(char *name, drvdata_hdl_t handle)
+{
+	struct vm *vm;
+	ASSERT(handle < driver_data_size);
+
+	vm = vm_find_by_name(name);
+	if (vm == NULL) {
+		return NULL;
+	}
+	return (void *)((ulong)(vm->driver_data) + handle);
+}
+
 unsigned int
 vm_get_vcpu_count()
 {
@@ -95,33 +114,6 @@ vm_get_name()
 	return current->vm->name;
 }
 
-static u32
-vm_create_id(char *name)
-{
-	char buf[VM_NAME_BUF_SIZE];
-	u32 id = 0;
-	int i = 0;
-	vmmerr_t err;
-
-	if (strcmp(name, VM0_NAME) == 0) {
-		return 0;
-	}
-
-	while ((err = param_get_str("vm", i++, buf, VM_NAME_BUF_SIZE))
-	       != VMMERR_NOTEXIST) {
-		if (err != VMMERR_SUCCESS) {
-			continue;
-		}
-		if (strcmp(buf, VM0_NAME) != 0) {
-			id++;
-		}
-		if (strcmp(buf, name) == 0) {
-			return id;
-		}
-	}
-	return (u32)-1;
-}
-
 u32
 vm_get_id(void)
 {
@@ -134,93 +126,55 @@ vm_vbsp_apic_id(void)
 	return current->vm->vbsp_apic_id;
 }
 
-static struct vm *
+struct vm *
+vm_find(void)
+{
+	struct vm *vm;
+	struct vm_config *vm_config;
+	struct cpu_config *cpu_config;
+	char *name = NULL;
+
+	if (cpu_is_bsp()) {
+		return vm0;
+	}
+
+	LIST2_FOREACH(vm_config_list.head, vm_config_list, vm_config) {
+		LIST2_FOREACH(vm_config->cpu_list, cpu_list, cpu_config) {
+			if (cpu_config->apic_id == get_apic_id()) {
+				name = vm_config->name;
+				goto found;
+			}
+		}
+	}
+	return vm0;
+found:
+	LIST2_FOREACH(vm_list, vm_list, vm) {
+		if (strcmp(name, vm->name) == 0) {
+			return vm;
+		}
+	}
+	return NULL;
+}
+
+struct vm *
 vm_find_by_name(char *name)
 {
 	struct vm *vm;
 
-	spinlock_lock(&vm_list_lock);
 	LIST2_FOREACH(vm_list, vm_list, vm) {
-		if (strcmp(vm->name, name) == 0) {
-			break;
+		if (strcmp(name, vm->name) == 0) {
+			return vm;
 		}
 	}
-	spinlock_unlock(&vm_list_lock);
-	return vm;
-}
-
-static vmmerr_t
-vm_get_name_from_param(char *buf, size_t size)
-{
-	char name[VM_NAME_BUF_SIZE];
-	char param_name[VM_NAME_BUF_SIZE + 4];
-	u32 apic_id;
-	int i = 0;
-	int j;
-	vmmerr_t err;
-
-	ASSERT(sizeof(u32) == sizeof(apic_id_t));
-
-	if (cpu_is_bsp()) {
-		/*
-		 * CPU0 is always VM0.
-		 */
-		snprintf(buf, size, "%s", VM0_NAME);
-		return VMMERR_SUCCESS;
-	}
-
-	while ((err = param_get_str("vm", i++, name, VM_NAME_BUF_SIZE))
-	       != VMMERR_NOTEXIST) {
-		if (err != VMMERR_SUCCESS) {
-			printf("Failed to get vm(%d) paramater. err 0x%x\n",
-			       i - 1, err);
-			continue;
-		}
-		snprintf(param_name, VM_NAME_BUF_SIZE + 4, "%s.cpu", name);
-		j = 0;
-		while ((err = param_get_u32(param_name, j++, &apic_id))
-		       != VMMERR_NOTEXIST) {
-			if (err != VMMERR_SUCCESS) {
-				printf("Failed to get %s(%d) paramater. "
-				       "err 0x%x\n",
-				       param_name, j - 1, err);
-				continue;
-			}
-			if (apic_id == get_apic_id()) {
-				snprintf(buf, size, "%s", name);
-				return VMMERR_SUCCESS;
-			}
-		}
-	}
-
-	/*
-	 * CPUs not specified by <vmname>.cpu parameter are VM0.
-	 */
-	snprintf(buf, size, "%s", VM0_NAME);
-	return VMMERR_SUCCESS;
-}
-
-struct vm *
-vm_find(void)
-{
-	char name[VM_NAME_BUF_SIZE];
-	vmmerr_t err;
-
-	err = vm_get_name_from_param(name, VM_NAME_BUF_SIZE);
-	if (err != VMMERR_SUCCESS) {
-		return NULL;
-	}
-	return vm_find_by_name(name);
+	return NULL;
 }
 
 bool
 vm_current_is_vbsp(void)
 {
-	char name[VM_NAME_BUF_SIZE];
-	char param_name[VM_NAME_BUF_SIZE + 4];
-	u32 apic_id;
-	vmmerr_t err;
-	int i;
+	struct vm_config *vm_config;
+	struct cpu_config *cpu_config;
+	bool is_vbsp;
 
 	if (cpu_is_bsp()) {
 		/*
@@ -229,46 +183,25 @@ vm_current_is_vbsp(void)
 		return true;
 	}
 
-	err = vm_get_name_from_param(name, VM_NAME_BUF_SIZE);
-	if (err != VMMERR_SUCCESS) {
-		return false;
-	}
-
-	if (strcmp(name, VM0_NAME) == 0) {
-		return false;
-	}
-
-	snprintf(param_name, VM_NAME_BUF_SIZE + 4, "%s.cpu", name);
-
-	i = 0;
-	while ((err = param_get_u32(param_name, i++, &apic_id))
-	       != VMMERR_NOTEXIST) {
-		if (err != VMMERR_SUCCESS) {
-			if (err != VMMERR_NOTEXIST) {
-				printf("Failed to get %s(%d) paramater. "
-				       "err 0x%x\n", param_name, i - 1, err);
-				continue;
+	LIST2_FOREACH(vm_config_list.head, vm_config_list, vm_config) {
+		is_vbsp = true;
+		LIST2_FOREACH(vm_config->cpu_list, cpu_list, cpu_config) {
+			if (cpu_config->apic_id == get_apic_id()) {
+				return is_vbsp;
 			}
-		}
-		if (apic_id != get_bsp_apic_id()) {
-			break;
+			is_vbsp = false;
 		}
 	}
-	if (err == VMMERR_NOTEXIST) {
-		panic("Can't found the self apic_id.\n");
-	}
-	if (apic_id == get_apic_id()) {
-		return true;
-	} else {
-		return false;
-	}
+	return false;
 }
 
-struct vm *
-vm_new(void)
+static struct vm *
+vm_new(u32 id, struct vm_config *vm_config)
 {
 	struct vm *vm;
-	vmmerr_t err;
+	struct cpu_config *cpu_config;
+
+	ASSERT(cpu_is_bsp());
 
 	new_vm_called = 1;
 
@@ -278,38 +211,57 @@ vm_new(void)
 	}
 	memset(vm, 0, sizeof(struct vm) + driver_data_size);
 
-	err = vm_get_name_from_param(vm->name, VM_NAME_BUF_SIZE);
-	if (err != VMMERR_SUCCESS) {
-		free(vm);
-		return NULL;
-	}
-
-	vm->id = vm_create_id(vm->name);
+	vm->id = id;
+	snprintf(vm->name, VM_NAME_BUF_SIZE, "%s", vm_config->name);
 
 	spinlock_init(&vm->vcpu_list_lock);
-	LIST2_HEAD_INIT(vm->vcpu_list);
+	LIST2_HEAD_INIT(vm->vcpu_list, vcpu_list);
 
-	spinlock_lock(&vm_list_lock);
 	LIST2_ADD(vm_list, vm_list, vm);
-	vm_num++;
-	spinlock_unlock(&vm_list_lock);
 
-	vm->vbsp_apic_id = get_apic_id();
+	cpu_config = LIST2_NEXT(vm_config->cpu_list, cpu_list,
+				(struct cpu_config *)NULL);
+	if (cpu_config) {
+		vm->vbsp_apic_id = cpu_config->apic_id;
+	} else {
+		vm->vbsp_apic_id = get_apic_id();
+	}
 
 	wait_init(&vm->wait);
 	return vm;
 }
 
+void
+vm_create_all(void)
+{
+	struct vm_config *vm_config;
+	u32 id = 0;
+
+	ASSERT(cpu_is_bsp());
+
+	vm0 = vm_new(id++, &vm0_config);
+	if (vm0 == NULL) {
+		panic("Can't alloc vm0");
+	}
+
+	LIST2_FOREACH(vm_config_list.head, vm_config_list, vm_config) {
+		if (strcmp(vm_config->name, VM0_NAME) == 0) {
+			continue;
+		}
+		vm_new(id++, vm_config);
+	}
+}
+
 void
 vm_dump(void)
 {
+	static spinlock_t vm_dump_lock = SPINLOCK_INITIALIZER;
 	struct vm *vm;
 	struct vcpu *vcpu;
 	int index = 0;
 	phys_t gphys;
 	phys_t len;
 	u32 type;
-	static spinlock_t vm_dump_lock = SPINLOCK_INITIALIZER;
 
 	spinlock_lock(&vm_dump_lock);
 	vm = current->vm;
@@ -319,7 +271,8 @@ vm_dump(void)
 		printf(" 0x%x", vcpu->apic_id);
 	}
 	printf("\n");
-	while (vm->gmm.get_mem_map(index++, &gphys, &len, &type, NULL)) {
+
+	while (gmm_get_mem_map(index++, &gphys, &len, &type, NULL)) {
 		printf("    %016llx-%016llx 0x%x\n",
 		       gphys, gphys + len - 1, type);
 	}
@@ -346,6 +299,46 @@ vm_resetting(void)
 	return current->vm->resetting;
 }
 
+int
+vm0_boot_method(u8 drive, u8 *boot_drive)
+{
+	static bool boot_once = false;
+	int method;
+
+	if (!boot_once) {
+		bios_boot_drive = drive;
+	}
+
+	if (vm0_config.boot_int18) {
+		if (boot_once) {
+			printf("Boot from 0x%x drive.\n", 0x80);
+			method = 1;
+			*boot_drive = 0x80;
+		} else {
+			printf("Boot using int18.\n");
+			method = 0;
+		}
+	} else {
+		if (vm0_config.boot_drive != 0x00) {
+			*boot_drive = vm0_config.boot_drive;
+			printf("Boot from 0x%x drive.\n", *boot_drive);
+			method = 1;
+		} else {
+			printf("Boot from 0x%x drive.\n",
+			       bios_boot_drive);
+			*boot_drive = bios_boot_drive;
+			method = 1;
+		}
+		if (vm0_config.boot_partition) {
+			printf("Use loader of active partition\n");
+			*boot_drive = bios_boot_drive;
+			method = 2;
+		}
+	}
+	boot_once = true;
+	return method;
+}
+
 vmmerr_t
 vm_reset(void)
 {
@@ -373,26 +366,24 @@ vm_reset(void)
 
 	vm_wait_other_cpus();
 
-	if (vm->gmm.clear_mem) {
-		vm->gmm.clear_mem();
+	if (current->gmm.clear_mem) {
+		current->gmm.clear_mem();
 	}
 
-	if (vm_get_id() == 0) {
-		setup_guestboot();
-	} else {
-		if (gbios_load() != VMMERR_SUCCESS) {
-			apic_enter_wait_for_sipi_state();
-		}
-	}
+	call_initfunc("reset");
 
-	call_initfunc("resetvm");
-
-	vm_wait_other_cpus();
-	extint_reset_apic();
-	vm_wait_other_cpus();
+	exint_reset_apic();
 
 	vm->resetting = false;
 
+	if (vm_get_id() == 0) {
+		vmmcall_boot_enable(reload_os_thread, NULL);
+	} else {
+		if (gbios_load() != VMMERR_SUCCESS) {
+			current->vmctl.init_signal ();
+		}
+	}
+
 	return VMMERR_SUCCESS;
 }
 
@@ -407,18 +398,16 @@ vm_shutdown(void)
 	current->vm->shutdown = true;
 
 	if (vm_get_id() != 0) {
-		freeze();
+		halt_cpu();
 	}
 
 	for (;;) {
 		all_shutdown = true;
-		spinlock_lock(&vm_list_lock);
 		LIST2_FOREACH(vm_list, vm_list, vm) {
 			if (vm->shutdown == false) {
 				all_shutdown = false;
 			}
 		}
-		spinlock_unlock(&vm_list_lock);
 		if (all_shutdown) {
 			break;
 		}
@@ -426,5 +415,5 @@ vm_shutdown(void)
 	}
 
 	acpi_poweroff();
-	freeze();
+	halt_cpu();
 }
